package org.freecompany.util.xml.dom2sax;

import java.util.logging.Logger;
import org.freecompany.util.multicaster.Notification;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.AttributesImpl;

import static javax.xml.XMLConstants.XMLNS_ATTRIBUTE_NS_URI;
import static org.freecompany.util.multicaster.NotificationMulticaster.logging;

public class Dom2Sax {

	protected final Notification logger = logging( Dom2Sax.class);
	protected ContentHandler content;
	protected LexicalHandler lexical;

	public void setContentHandler( final ContentHandler content) {
		this.content = content;
	}

	public void setLexicalHandler( final LexicalHandler lexical) {
		this.lexical = lexical;
	}

	private String correct( String value) {
		return value == null ? "" : value;
	}

	/**
	 * Converts a DOM subtree into SAX events.
	 * 
	 * @param root the root of the DOM subtree
	 * @throws SAXException when the underlying content content does
	 */
	public void convert( final Node root) throws SAXException {
		logger.debug( "Converting node ''{0}''.", root);
		switch ( root.getNodeType()) {
		case Node.DOCUMENT_NODE:
			content.startDocument();
			convert((( Document) root).getDocumentElement());
			content.endDocument();
			break;

		case Node.ELEMENT_NODE:
			final NamedNodeMap attrMap = root.getAttributes();
			final AttributesImpl attrs = new AttributesImpl();
			final int n = attrMap.getLength();
			for ( int i = 0; i < n; i++) {
				final Node attr = attrMap.item( i);
				final String ns = attr.getNamespaceURI();
				logger.debug( "Attribute has namespace ''{0}''.", ns);
				if ( "xmlns".equals( attr.getLocalName())) {
					logger.debug( "Mapping implicit namespace ''{0}''.", attr.getNodeValue());
					content.startPrefixMapping( "", attr.getNodeValue());
				} else if ( XMLNS_ATTRIBUTE_NS_URI.equals( ns)) {
					logger.debug( "Mapping explicit namespace ''{0}'' to ''{1}''.", attr.getNodeValue(), attr.getLocalName());
					content.startPrefixMapping( attr.getLocalName(), attr.getNodeValue());
				} else {
					logger.debug( "Adding attribute ''{0}'' with value ''{1}''.", attr.getNodeName(), attr.getNodeValue());
					final String name = attr.getLocalName();
					attrs.addAttribute( ns, name == null ? attr.getNodeName() : name, attr.getNodeName(), null, attr.getNodeValue());
				}
			}
			String namespace = correct( root.getNamespaceURI());
			String local = correct( root.getLocalName());
			String qname = correct( root.getNodeName());
			logger.debug( "Detected relevant namespace ''{0}'' for local name ''{1}'' qname ''{2}''.", namespace, local, qname);
			content.startElement( namespace, local, qname, attrs);
			for ( Node child = root.getFirstChild(); child != null; child = child.getNextSibling()) convert( child);
			content.endElement( namespace, local, qname);
			break;

		case Node.CDATA_SECTION_NODE:
			// XXX: Ignore if no SAX2 support.
			if ( lexical != null) {
				final char[] cdata = root.getNodeValue().toCharArray();
				lexical.startCDATA();
				content.characters( cdata, 0, cdata.length);
				lexical.endCDATA();
			}
			break;

		case Node.TEXT_NODE:
			final char[] characters = root.getNodeValue().toCharArray();
			content.characters( characters, 0, characters.length);
			break;

		default:
			// skip it
		}
	}
}
